from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.branch_base import BranchBaseAttributes


class EngineRootAttributes(BranchBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Engine Root"
	acme_certificates_folder = Attribute('ACME Certificates Folder', min_version='17.2')
	acme_enabled = Attribute('ACME Enabled', min_version='17.2')
	adaptable_script_max_file_size = Attribute('Adaptable Script Max File Size', min_version='19.3')
	advanced_key_protect = Attribute('Advanced Key Protect', min_version='18.1')
	allow_acme_folder_creation = Attribute('Allow ACME Folder Creation', min_version='17.2')
	authentication_scheme = Attribute('Authentication Scheme')
	automatically_apply_filters = Attribute('Automatically Apply Filters', min_version='21.3')
	disabled_features = Attribute('Disabled Features', min_version='21.1')
	flow_process_expiry = Attribute('Flow Process Expiry', min_version='21.2')
	key_pair_renewal_flow = Attribute('Key Pair Renewal Flow', min_version='20.2')
	known_cloud_service = Attribute('Known Cloud Service', min_version='17.1')
	known_cloud_service_region = Attribute('Known Cloud Service Region', min_version='17.1')
	large_tree_support = Attribute('Large Tree Support')
	managed_by_options = Attribute('Managed By Options')
	monitored_uri_verification_engine_info = Attribute('Monitored URI Verification Engine Info')
	monitored_uri_verification_identifier = Attribute('Monitored URI Verification Identifier')
	monitored_uri_verification_last_check = Attribute('Monitored URI Verification Last Check')
	monitored_uri_verification_last_notification = Attribute('Monitored URI Verification Last Notification')
	monitored_uri_verification_uri_identifier = Attribute('Monitored URI Verification URI Identifier')
	oam_enabled = Attribute('OAM Enabled')
	oam_headervar = Attribute('OAM HeaderVar')
	oam_logout_url = Attribute('OAM Logout Url', min_version='20.1')
	operating_environment = Attribute('Operating Environment', min_version='15.1')
	options = Attribute('Options', min_version='20.4')
	pendo_events_last_sent = Attribute('Pendo Events Last Sent', min_version='21.4')
	require_password = Attribute('Require Password')
	revocation_check_issuer_dn = Attribute('Revocation Check Issuer DN')
	revocation_check_issuer_identifier = Attribute('Revocation Check Issuer Identifier')
	secret_key_renewal_flow = Attribute('Secret Key Renewal Flow', min_version='20.2')
	tn_aperture_plugin_vault_id = Attribute('TN Aperture Plugin Vault Id', min_version='18.1')
	tn_public_cas = Attribute('TN Public CAs', min_version='18.4')
	tn_widget_vault = Attribute('TN Widget Vault', min_version='18.1')
	teams_enabled = Attribute('Teams Enabled', min_version='20.1')
	teams_options = Attribute('Teams Options', min_version='20.2')
	teams_required = Attribute('Teams Required', min_version='20.1')
	time_stamping_certificate_dn = Attribute('Time Stamping Certificate DN', min_version='20.3')
	time_stamping_proxy_host_urls = Attribute('Time Stamping Proxy Host URLs', min_version='20.3')
	time_stamping_proxy_options = Attribute('Time Stamping Proxy Options', min_version='20.3')
	timeout = Attribute('Timeout', min_version='21.1')
	trustnet_critical_event_notification_emails = Attribute('TrustNet Critical Event Notification Emails', min_version='18.2')
	trustnet_emergency_event_notification_emails = Attribute('TrustNet Emergency Event Notification Emails', min_version='18.2')
	trustnet_heavy_job_schedule = Attribute('TrustNet Heavy Job Schedule', min_version='18.2')
	trustnet_info_event_notification_emails = Attribute('TrustNet Info Event Notification Emails', min_version='18.2')
	trustnet_last_security_event_timestamp = Attribute('TrustNet Last Security Event Timestamp', min_version='18.2')
	trustnet_light_job_schedule = Attribute('TrustNet Light Job Schedule', min_version='18.2')
	trustnet_placement_rules = Attribute('TrustNet Placement Rules', min_version='19.1')
	trustnet_sanitize_emails = Attribute('TrustNet Sanitize Emails', min_version='18.2')
	trustnet_task_last_run = Attribute('TrustNet Task Last Run', min_version='19.1')
	trustnet_trusted_ca = Attribute('TrustNet Trusted CA', min_version='18.2')
	trustnet_untrusted_ca = Attribute('TrustNet Untrusted CA', min_version='18.2')
	trustnet_warning_event_notification_emails = Attribute('TrustNet Warning Event Notification Emails', min_version='18.2')
	trusted_websdk_modules = Attribute('Trusted WebSDK Modules')
	usage_tracking = Attribute('Usage Tracking', min_version='18.1')
	websdk_options = Attribute('WebSDK Options')
	whitelisted_elevation_commands = Attribute('Whitelisted Elevation Commands', min_version='17.1')
	workflow_interval = Attribute('Workflow Interval', min_version='18.2')
	workflow_last_check = Attribute('Workflow Last Check', min_version='18.2')



from typing import List, Dict, Union
from pytpp.api.api_base import API, APIResponse, api_response_property
from pytpp.properties.response_objects.codesign import CodeSign


class _Codesign:
    def __init__(self, api_obj):
        self.AddAdministrator = self._AddAdministrator(api_obj=api_obj)
        self.AddApplicationAdministrator = self._AddApplicationAdministrator(api_obj=api_obj)
        self.AddProjectAdministrator = self._AddProjectAdministrator(api_obj=api_obj)
        self.AddProjectApprover = self._AddProjectApprover(api_obj=api_obj)
        self.CountReferences = self._CountReferences(api_obj=api_obj)
        self.CreateApplication = self._CreateApplication(api_obj=api_obj)
        self.CreateApplicationCollection = self._CreateApplicationCollection(api_obj=api_obj)
        self.CreateEnvironment = self._CreateEnvironment(api_obj=api_obj)
        self.CreateProject = self._CreateProject(api_obj=api_obj)
        self.CreateTemplate = self._CreateTemplate(api_obj=api_obj)
        self.DeleteApplication = self._DeleteApplication(api_obj=api_obj)
        self.DeleteApplicationCollection = self._DeleteApplicationCollection(api_obj=api_obj)
        self.DeleteEnvironment = self._DeleteEnvironment(api_obj=api_obj)
        self.DeleteProject = self._DeleteProject(api_obj=api_obj)
        self.DeleteTemplate = self._DeleteTemplate(api_obj=api_obj)
        self.EnumerateApplications = self._EnumerateApplications(api_obj=api_obj)
        self.EnumerateApplicationCollections = self._EnumerateApplicationCollections(api_obj=api_obj)
        self.EnumerateProjects = self._EnumerateProjects(api_obj=api_obj)
        self.EnumerateReferences = self._EnumerateReferences(api_obj=api_obj)
        self.EnumerateTemplates = self._EnumerateTemplates(api_obj=api_obj)
        self.GetApplication = self._GetApplication(api_obj=api_obj)
        self.GetApplicationCollection = self._GetApplicationCollection(api_obj=api_obj)
        self.GetApplicationCollectionMembers = self._GetApplicationCollectionMembers(api_obj=api_obj)
        self.GetApplicationCollectionMemberDNs = self._GetApplicationCollectionMemberDNs(api_obj=api_obj)
        self.GetEnvironment = self._GetEnvironment(api_obj=api_obj)
        self.GetGlobalConfiguration = self._GetGlobalConfiguration(api_obj=api_obj)
        self.GetObjectRights = self._GetObjectRights(api_obj=api_obj)
        self.GetRight = self._GetRight(api_obj=api_obj)
        self.GetTemplate = self._GetTemplate(api_obj=api_obj)
        self.GetTrusteeRights = self._GetTrusteeRights(api_obj=api_obj)
        self.RemoveAdministrator = self._RemoveAdministrator(api_obj=api_obj)
        self.RemoveApplicationAdministrator = self._RemoveApplicationAdministrator(api_obj=api_obj)
        self.RemoveProjectAdministrator = self._RemoveProjectAdministrator(api_obj=api_obj)
        self.RenameApplication = self._RenameApplication(api_obj=api_obj)
        self.RenameApplicationCollection = self._RenameApplicationCollection(api_obj=api_obj)
        self.RenameProject = self._RenameProject(api_obj=api_obj)
        self.RenameTemplate = self._RenameTemplate(api_obj=api_obj)
        self.SetGlobalConfiguration = self._SetGlobalConfiguration(api_obj=api_obj)
        self.UpdateApplication = self._UpdateApplication(api_obj=api_obj)
        self.UpdateApplicationCollection = self._UpdateApplicationCollection(api_obj=api_obj)
        self.UpdateEnvironment = self._UpdateEnvironment(api_obj=api_obj)
        self.UpdateProject = self._UpdateProject(api_obj=api_obj)
        self.UpdateProjectStatus = self._UpdateProjectStatus(api_obj=api_obj)
        self.UpdateTemplate = self._UpdateTemplate(api_obj=api_obj)

    class _AddAdministrator(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/AddAdministrator')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _AddApplicationAdministrator(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/AddApplicationAdministrator')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _AddProjectAdministrator(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/AddProjectAdministrator')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _AddProjectApprover(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/AddProjectApprover')
            
        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }
            
            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')
                    
            return _Response(response=self._post(data=body))
        

    class _AddPreApproval(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/AddPreApproval')

        def post(self, dn: str, comment: str, user: str, hours: int = None, ip_address: str = None,
                 signing_executable: str = None, single_use: bool = None):
            body = {
                'Dn': dn,
                'Comment': comment,
                'Hours': hours,
                'IPAddress': ip_address,
                'SigningExecutable': signing_executable,
                'SingleUse': single_use,
                'User': user
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _CountReferences(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/CountReferences')

        def post(self, application: dict = None, application_collection: dict = None):
            body = {
                'Application': application,
                'ApplicationCollection': application_collection
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def count(self) -> int:
                    return self._from_json(key='Count')

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _CreateApplication(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/CreateApplication')

        def post(self, dn: str):
            body = {
                'Dn': dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application(self):
                    return CodeSign.Application(self._from_json(key='Application'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _CreateApplicationCollection(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/CreateApplicationCollection')

        def post(self, dn: str):
            body = {
                'Dn': dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application(self):
                    return CodeSign.ApplicationCollection(self._from_json(key='ApplicationCollection'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _CreateEnvironment(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/CreateEnvironment')

        def post(self, dn: str, environment_name: str, project: Dict[str, Union[str, int]],
                 template: List[Dict[str, str]], template_dn: str = None):
            body = {
                'Dn': dn,
                'EnvironmentName': environment_name,
                'Project': project,
                'Template': template,
                'TemplateDn': template_dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def apple_environment(self) -> str:
                    return CodeSign.AppleEnvironment(self._from_json(key='AppleEnvironment'))

                @property
                @api_response_property()
                def csp_environment(self) -> str:
                    return CodeSign.CSPEnvironment(self._from_json(key='CSPEnvironment'))

                @property
                @api_response_property()
                def dot_net_environment(self) -> str:
                    return CodeSign.DotNetEnvironment(self._from_json(key='DotNetEnvironment'))

                @property
                @api_response_property()
                def gpg_environment(self) -> str:
                    return CodeSign.GPGEnvironment(self._from_json(key='GPGEnvironment'))

                @property
                @api_response_property()
                def key_pair_environment(self) -> str:
                    return CodeSign.KeyPairEnvironment(self._from_json(key='KeyPairEnvironment'))

                @property
                @api_response_property()
                def apple_template(self) -> str:
                    return CodeSign.AppleTemplate(self._from_json(key='AppleTemplate'))

                @property
                @api_response_property()
                def csp_template(self) -> str:
                    return CodeSign.CSPTemplate(self._from_json(key='EnviCSPTemplateronment'))

                @property
                @api_response_property()
                def dot_net_template(self) -> str:
                    return CodeSign.DotNetTemplate(self._from_json(key='DotNetTemplate'))

                @property
                @api_response_property()
                def gpg_template(self) -> str:
                    return CodeSign.GPGTemplate(self._from_json(key='GPGTemplate'))

                @property
                @api_response_property()
                def key_pair_environment(self) -> str:
                    return CodeSign.KeyPairTemplate(self._from_json(key='KeyPairTemplate'))

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _CreateProject(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/CreateProject')

        def post(self, dn: str):
            body = {
                'Dn': dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def project(self):
                    return CodeSign.Project(self._from_json(key='Project'))

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _CreateTemplate(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/CreateTemplate')

        def post(self, dn: str, template_type: str, per_user: bool):
            body = {
                'Dn': dn,
                'TemplateType': template_type,
                'PerUser': per_user
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def certificate_template(self):
                    return CodeSign.CertificateTemplate(self._from_json(key='CertificateTemplate'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _DeleteApplication(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/DeleteApplication')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _DeleteApplicationCollection(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/DeleteApplicationCollection')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn'  : dn,
                'Guid': guid,
                'Id'  : id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _DeleteEnvironment(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/DeleteEnvironment')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _DeleteProject(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/DeleteProject')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn'  : dn,
                'Guid': guid,
                'Id'  : id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _DeleteTemplate(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/DeleteTemplate')

        def post(self, dn: str = None, force: bool = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Force': force,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _EnumerateApplications(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/EnumerateApplications')

        def post(self, filter: str = None):
            body = {
                'Filter': filter
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def applications(self):
                    return [
                        CodeSign.Application(a) for a in self._from_json(key='Applications')
                    ]

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _EnumerateApplicationCollections(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/EnumerateApplicationCollections')

        def post(self, filter: str = None):
            body = {
                'Filter': filter
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application_collections(self):
                    return [
                        CodeSign.ApplicationCollection(ac)
                        for ac in self._from_json(key='ApplicationCollections')
                    ]

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _EnumerateProjects(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/EnumerateProjects')

        def post(self, filter: str = None, rights: int = None):
            body = {
                'Filter': filter,
                'Rights': rights
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def projects(self):
                    return [CodeSign.Project(p) for p in self._from_json(key='Projects')]

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _EnumerateReferences(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/EnumerateReferences')

        def post(self, application: dict = None, application_collection: dict = None,
                 application_dn: str = None, application_guid: str = None,
                 collection_dn: str = None, collection_guid: str = None):
            body = {
                'Application': application,
                'ApplicationCollection': application_collection,
                'ApplicationDn': application_dn,
                'ApplicationGuid': application_guid,
                'CollectionDn': collection_dn,
                'CollectionGuid': collection_guid
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def reference_dns(self) -> List[str]:
                    return self._from_json(key='ReferenceDNs')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')


            return _Response(response=self._post(data=body))

    class _EnumerateTemplates(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/EnumerateTemplates')

        def post(self, filter: str = None):
            body = {
                'Filter': filter
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def certificate_templates(self):
                    return [
                        CodeSign.CertificateTemplate(t)
                        for t in self._from_json(key='CertificateTemplates')
                    ]

                @property
                @api_response_property()
                def csp_templates(self):
                    return [
                        CodeSign.CSPTemplate(t)
                        for t in self._from_json(key='CSPTemplates')
                    ]

                @property
                @api_response_property()
                def dot_net_templates(self):
                    return [
                        CodeSign.DotNetTemplate(t)
                        for t in self._from_json(key='DotNetTemplates')
                    ]

                @property
                @api_response_property()
                def gpg_templates(self):
                    return [
                        CodeSign.GPGTemplate(t)
                        for t in self._from_json(key='GPGTemplates')
                    ]

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetApplication(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetApplication')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application(self):
                    return CodeSign.Application(self._from_json(key='Application'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetApplicationCollection(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetApplicationCollection')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn'  : dn,
                'Guid': guid,
                'Id'  : id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application_collection(self):
                    return CodeSign.ApplicationCollection(self._from_json(key='ApplicationCollection'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetApplicationCollectionMembers(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetApplicationCollectionMembers')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn'  : dn,
                'Guid': guid,
                'Id'  : id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application_collection(self):
                    return CodeSign.ApplicationCollection(self._from_json(key='ApplicationCollection'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetApplicationCollectionMemberDNs(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetApplicationCollectionMemberDNs')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn'  : dn,
                'Guid': guid,
                'Id'  : id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def application_collection(self):
                    return CodeSign.ApplicationCollection(self._from_json(key='ApplicationCollection'))

                @property
                @api_response_property()
                def application_collection_dns(self) -> List[str]:
                    return self._from_json(key='ApplicationCollectionDNs')

                @property
                @api_response_property()
                def application_dns(self) -> List[str]:
                    return self._from_json(key='ApplicationDNs')

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetEnvironment(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetEnvironment')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def certificate_environment(self):
                    return CodeSign.CertificateEnvironment(self._from_json(key='CertificateEnvironment'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetGlobalConfiguration(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetGlobalConfiguration')

        def get(self):
            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def global_configuration(self):
                    return CodeSign.GlobalConfiguration(self._from_json(key='GlobalConfiguration'))

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._get())

    class _GetObjectRights(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetObjectRights')

        def post(self, dn: str):
            body = {
                'Dn': dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def rights_list(self):
                    return [CodeSign.RightsKeyValue(r) for r in self._from_json(key='RightsList')]

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetProject(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetProject')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn'  : dn,
                'Guid': guid,
                'Id'  : id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def project(self):
                    return CodeSign.Project(self._from_json(key='Project'))

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetRight(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetRight')

        def post(self, dn: str):
            body = {
                'Dn': dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def rights(self):
                    return CodeSign.Rights(self._from_json(key='Rights'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetTemplate(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetTemplate')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def certificate_template(self):
                    return CodeSign.CertificateTemplate(self._from_json(key='CertificateTemplate'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _GetTrusteeRights(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/GetTrusteeRights')

        def post(self, dn: str):
            body = {
                'Dn': dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def rights_list(self):
                    return [CodeSign.RightsKeyValue(r) for r in self._from_json(key='RightsList')]

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RemoveAdministrator(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RemoveAdministrator')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RemoveApplicationAdministrator(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RemoveApplicationAdministrator')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RemoveProjectAdministrator(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RemoveProjectAdministrator')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RemoveProjectApprover(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RemoveProjectApprover')

        def post(self, trustee: str):
            body = {
                'Trustee': trustee
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RenameApplication(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RenameApplication')

        def post(self, dn: str, new_dn: str):
            body = {
                'Dn': dn,
                'NewDn': new_dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RenameApplicationCollection(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RenameApplicationCollection')

        def post(self, dn: str, new_dn: str):
            body = {
                'Dn'   : dn,
                'NewDn': new_dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RenameProject(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RenameProject')

        def post(self, new_dn: str, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id,
                'NewDn': new_dn
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _RenameTemplate(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/RenameTemplate')

        def post(self, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _SetGlobalConfiguration(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/SetGlobalConfiguration')

        def post(self, global_configuration: dict):
            body = {
                'GlobalConfiguration': global_configuration
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _UpdateApplication(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/UpdateApplication')

        def post(self, application: dict):
            body = {
                'Application': application
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _UpdateApplicationCollection(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/UpdateApplicationCollection')

        def post(self, application_collection: dict):
            body = {
                'ApplicationCollection': application_collection
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _UpdateEnvironment(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/UpdateEnvironment')

        def post(self, certificate_environment: dict):
            body = {
                'CertificateEnvironment': certificate_environment
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def certificate_environment(self):
                    return CodeSign.CertificateEnvironment(self._from_json(key='CertificateEnvironment'))

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _UpdateProject(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/UpdateProject')

        def post(self, project: dict):
            body = {
                'Project': project
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _UpdateProjectStatus(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/UpdateProjectStatus')

        def post(self, project_status: int, dn: str = None, guid: str = None, id: int = None):
            body = {
                'Dn': dn,
                'Guid': guid,
                'Id': id,
                'ProjectStatus': project_status,
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))

    class _UpdateTemplate(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Codesign/UpdateTemplate')

        def post(self, dn: str, certificate_template: dict, object_naming_pattern: str = None):
            body = {
                'Dn': dn,
                'CertificateTemplate': certificate_template,
                'ObjectNamingPattern': object_naming_pattern
            }

            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def error(self) -> str:
                    return self._from_json(key='Error')

                @property
                @api_response_property()
                def result(self):
                    return CodeSign.ResultCode(self._from_json(key='Result'))

                @property
                @api_response_property()
                def success(self) -> bool:
                    return self._from_json(key='Success')

            return _Response(response=self._post(data=body))



from pytpp.attributes._helper import IterableMeta
from pytpp.attributes.credential_base import CredentialBaseAttributes


class GoogleCredentialAttributes(CredentialBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Google Credential"



from dataclasses import dataclass
from datetime import datetime
from typing import List


@dataclass
class Certificate:
    created_on: str
    dn: str
    guid: str
    name: str
    parent_dn: str
    schema_class: str
    x509: '_X509'
    links: 'List[Link]'


@dataclass
class Link:
    details: str
    next: str
    previous: str


@dataclass
class CSR:
    details: '_CSRDetails'
    enrollable: bool


@dataclass
class Policy:
    certificate_authority: '_LockedSingleValue'
    csr_generation: '_LockedSingleValue'
    management_type: '_LockedSingleValue'
    key_generation: '_LockedSingleValue'
    key_pair: '_LockedKeyPair'
    private_key_reuse_allowed: bool
    subj_alt_name_dns_allowed: bool
    subj_alt_name_email_allowed: bool
    subj_alt_name_ip_allowed: bool
    subj_alt_name_upn_allowed: bool
    subj_alt_name_uri_allowed: bool
    subject: '_LockedSubject'
    unique_subject_enforced: bool
    whitelisted_domains: list
    wildcards_allowed: bool


@dataclass
class CertificateDetails:
    c: str
    cn: str
    enhanced_key_usage: str
    issuer: str
    key_algorithm: str
    key_size: int
    key_usage: str
    l: str
    o: str
    ou: str
    public_key_hash: str
    s: str
    ski_key_identifier: str
    serial: str
    signature_algorithm: str
    signature_algorithm_oid: str
    store_added: str
    subject: str
    subject_alt_name_dns: str
    subject_alt_name_email: str
    subject_alt_name_ip: str
    subject_alt_name_upn: str
    subject_alt_name_uri: str
    thumbprint: str
    valid_from: 'datetime'
    valid_to: 'datetime'


@dataclass
class PreviousVersions:
    certificate_details: 'CertificateDetails'
    vault_id: int


@dataclass
class ProcessingDetails:
    in_error: bool
    stage: int
    status: str


@dataclass
class RenewalDetails:
    city: str
    country: str
    organization: str
    organizational_unit: str
    state: str
    subject: str
    subject_alt_name_dns: str
    subject_alt_name_email: str
    subject_alt_name_ip_address: str
    subject_alt_name_other_name_upn: str
    subject_alt_name_uri: str
    valid_from: 'datetime'
    valid_to: 'datetime'


@dataclass
class ValidationDetails:
    last_validation_state_update: str
    validation_state: str


@dataclass
class SslTls:
    host: str
    ip_address: str
    port: int
    result: '_SslTlsResult'
    sources: list


@dataclass
class File:
    installation: str
    performed_on: 'datetime'
    result: List[str]


@dataclass
class _SslTlsResult:
    chain: '_BitMaskValues'
    end_entity: '_BitMaskValues'
    id: int
    protocols: '_BitMaskValues'


@dataclass
class _BitMaskValues:
    bitmask: int
    values: List[str]


@dataclass
class _SANS:
    dns: str
    ip: str


@dataclass
class _X509:
    cn: str
    issuer: str
    key_algorithm: str
    key_size: str
    sans: str
    serial: str
    subject: str
    thumbprint: str
    valid_from: 'datetime'
    valid_to: 'datetime'


@dataclass
class _Compliant:
    compliant: bool


@dataclass
class _CompliantSingleValue(_Compliant):
    value: str


@dataclass
class _CompliantMultiValue(_Compliant):
    values: list


@dataclass
class _Locked:
    locked: bool


@dataclass
class _LockedSingleValue(_Locked):
    value: str


@dataclass
class _LockedMultiValue(_Locked):
    values: list


@dataclass
class _LockedKeyPair:
    key_algorithm: '_LockedSingleValue'
    key_size: '_LockedSingleValue'


@dataclass
class _LockedSubject:
    city: '_LockedSingleValue'
    country: '_LockedSingleValue'
    organization: '_LockedSingleValue'
    organizational_units: '_LockedMultiValue'
    state: '_LockedSingleValue'


@dataclass
class _CSRDetails:
    city: '_CompliantSingleValue'
    common_name: '_CompliantSingleValue'
    country: '_CompliantSingleValue'
    key_algorithm: '_CompliantSingleValue'
    key_size: '_CompliantSingleValue'
    organization: '_CompliantSingleValue'
    organizational_unit: '_CompliantMultiValue'
    private_key_reused: '_CompliantSingleValue'
    state: '_CompliantSingleValue'
    subj_alt_name_dns: '_CompliantMultiValue'
    subj_alt_name_email: '_CompliantMultiValue'
    subj_alt_name_ip: '_CompliantMultiValue'
    subj_alt_name_upn: '_CompliantMultiValue'
    subj_alt_name_uri: '_CompliantMultiValue'



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.top import TopAttributes


class X509RootCertificateAttributes(TopAttributes, metaclass=IterableMeta):
	__config_class__ = "X509 Root Certificate"
	aia_ca_issuer_location_description = Attribute('AIA CA Issuer Location Description')
	aia_ca_issuer_location_disabled = Attribute('AIA CA Issuer Location Disabled')
	aia_ca_issuer_location_mode = Attribute('AIA CA Issuer Location Mode')
	aia_ca_issuer_location_rank = Attribute('AIA CA Issuer Location Rank')
	aia_ca_issuer_location_status = Attribute('AIA CA Issuer Location Status')
	aia_ca_issuer_publishing_location = Attribute('AIA CA Issuer Publishing Location')
	aia_ocsp_location_description = Attribute('AIA OCSP Location Description')
	aia_ocsp_location_disabled = Attribute('AIA OCSP Location Disabled')
	aia_ocsp_location_mode = Attribute('AIA OCSP Location Mode')
	aia_ocsp_location_rank = Attribute('AIA OCSP Location Rank')
	aia_ocsp_location_status = Attribute('AIA OCSP Location Status')
	aia_ocsp_publishing_location = Attribute('AIA OCSP Publishing Location')
	cdp_aia_actions = Attribute('CDP AIA Actions')
	cdp_aia_in_error = Attribute('CDP AIA In Error')
	cdp_aia_status = Attribute('CDP AIA Status')
	cdp_aia_verification_now = Attribute('CDP AIA Verification Now')
	cdp_crl_metadata = Attribute('CDP CRL Metadata')
	cdp_crl_next_publish_percentage = Attribute('CDP CRL Next Publish Percentage')
	cdp_location_description = Attribute('CDP Location Description')
	cdp_location_disabled = Attribute('CDP Location Disabled')
	cdp_location_mode = Attribute('CDP Location Mode')
	cdp_location_rank = Attribute('CDP Location Rank')
	cdp_location_status = Attribute('CDP Location Status')
	cdp_publishing_location = Attribute('CDP Publishing Location')
	crl_delta_number = Attribute('CRL Delta Number')
	crl_delta_publishing_location = Attribute('CRL Delta Publishing Location')
	crl_issuer_dn = Attribute('CRL Issuer DN')
	crl_issuer_identifier = Attribute('CRL Issuer Identifier')
	crl_next_publish_date = Attribute('CRL Next Publish Date')
	crl_next_update = Attribute('CRL Next Update')
	crl_number = Attribute('CRL Number')
	crl_this_update = Attribute('CRL This Update')
	certificate_vault_id = Attribute('Certificate Vault Id')
	discovered_on = Attribute('Discovered On', min_version='15.4')
	escalation_notice_interval = Attribute('Escalation Notice Interval', min_version='20.4')
	escalation_notice_start = Attribute('Escalation Notice Start', min_version='20.4')
	expiration_notice_interval = Attribute('Expiration Notice Interval', min_version='20.4')
	expiration_notice_start = Attribute('Expiration Notice Start', min_version='20.4')
	monitored_uri_verification_engine_info = Attribute('Monitored URI Verification Engine Info')
	monitored_uri_verification_last_check = Attribute('Monitored URI Verification Last Check')
	monitored_uri_verification_last_notification = Attribute('Monitored URI Verification Last Notification')
	monitored_uri_verification_run_identifier = Attribute('Monitored URI Verification Run Identifier')
	monitored_uri_verification_uri_identifier = Attribute('Monitored URI Verification URI Identifier')
	trusted_status = Attribute('Trusted Status')



from pytpp.properties.response_objects.dataclasses import pki


class PKI:
    @staticmethod
    def PKI(response_object: dict):
        if not isinstance(response_object, dict):
            response_object = {}
        return pki.PKI(
            certificate_dn=response_object.get('CertificateDN'),
            certificate_guid=response_object.get('CertificateGuid'),
            pki_dn=response_object.get('PkiDn'),
            pki_guid=response_object.get('PkiGuid'),
        )

    @staticmethod
    def Certificate(response_object: dict):
        if not isinstance(response_object, dict):
            response_object = {}
        return pki.Certificate(
            city=response_object.get('City'),
            common_name=response_object.get('CommonName'),
            country=response_object.get('Country'),
            key_algorithm=response_object.get('KeyAlgorithm'),
            key_bit_size=response_object.get('KeyBitSize'),
            organization=response_object.get('Organization'),
            organizational_units=response_object.get('OrganizationalUnits'),
            sans=[PKI.SANS(sans) for sans in response_object.get('SANs', [])],
            state=response_object.get('State'),
        )

    @staticmethod
    def Installation(response_object: dict):
        if not isinstance(response_object, dict):
            response_object = {}
        return pki.Installation(
            credential_dn=response_object.get('CredentialDn'),
            host=response_object.get('Host'),
        )

    @staticmethod
    def SANS(response_object: dict):
        if not isinstance(response_object, dict):
            response_object = {}
        return pki.SANS(
            name=response_object.get('Name'),
            typename=response_object.get('TypeName'),
        )



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.top import TopAttributes


class CodeSigningProjectAttributes(TopAttributes, metaclass=IterableMeta):
	__config_class__ = "Code Signing Project"
	approval_submission_date = Attribute('Approval Submission Date', min_version='21.2')
	auditor = Attribute('Auditor', min_version='19.2')
	certificate_issue_flow_dn = Attribute('Certificate Issue Flow DN', min_version='19.2')
	certificate_owner = Attribute('Certificate Owner', min_version='19.2')
	code_signing_application_dn = Attribute('Code Signing Application DN', min_version='19.2')
	flow_instance_macro = Attribute('Flow Instance Macro', min_version='19.3')
	key_issue_flow_dn = Attribute('Key Issue Flow DN', min_version='19.2')
	key_owner = Attribute('Key Owner', min_version='19.2')
	key_use_approver = Attribute('Key Use Approver', min_version='19.2')
	key_user = Attribute('Key User', min_version='19.2')
	owner = Attribute('Owner', min_version='19.2')
	project_delete_flow_dn = Attribute('Project Delete Flow DN', min_version='21.1')
	purpose = Attribute('Purpose', min_version='19.2')
	risk_level = Attribute('Risk Level', min_version='19.2')
	status = Attribute('Status', min_version='19.2')



from typing import List
from pytpp.api.api_base import API, APIResponse, api_response_property
from pytpp.properties.response_objects.worfklow import Workflow


class _Workflow:
    def __init__(self, api_obj):
        self.Ticket = self._Ticket(api_obj=api_obj)

    class _Ticket:
        def __init__(self, api_obj):
            self.Create = self._Create(api_obj=api_obj)
            self.Delete = self._Delete(api_obj=api_obj)
            self.Details = self._Details(api_obj=api_obj)
            self.Enumerate = self._Enumerate(api_obj=api_obj)
            self.Exists = self._Exists(api_obj=api_obj)
            self.Status = self._Status(api_obj=api_obj)
            self.UpdateStatus = self._UpdateStatus(api_obj=api_obj)

        class _Create(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/Create')

            def post(self, object_dn: str, approvers: list, reason: str, workflow_dn: str, user_data: str = None):
                body = {
                    'ObjectDN': object_dn,
                    'Approvers': approvers,
                    'Reason': reason,
                    'UserData': user_data,
                    'WorkflowDN': workflow_dn
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def guid(self) -> str:
                        return self._from_json('GUID')

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))

        class _Delete(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/Delete')

            def post(self, guid: str):
                body = {
                    'GUID': guid
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))

        class _Details(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/Details')

            def post(self, guid: str):
                body = {
                    'GUID': guid
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def details(self):
                        return Workflow.Details(self._from_json())

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))

        class _Enumerate(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/Enumerate')

            def post(self, object_dn: str = None, user_data: str = None):
                body = {
                    'ObjectDN': object_dn,
                    'UserData': user_data
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def guids(self) -> List[str]:
                        return self._from_json('GUIDS')

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))

        class _Exists(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/Exists')

            def post(self, guid: str):
                body = {
                    'GUID': guid
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))

        class _Status(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/Status')

            def post(self, guid: str):
                body = {
                    'GUID': guid
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def status(self) -> str:
                        return self._from_json('Status')

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))

        class _UpdateStatus(API):
            def __init__(self, api_obj):
                super().__init__(api_obj=api_obj, url='/Workflow/Ticket/UpdateStatus')

            def post(self, guid: str, status: str, explanation: str = None, scheduled_start: str = None, scheduled_stop: str = None):
                body = {
                    'GUID': guid,
                    'Status': status,
                    'Explanation': explanation,
                    'ScheduledStart': scheduled_start,
                    'ScheduledStop': scheduled_stop
                }

                class _Response(APIResponse):
                    def __init__(self, response):
                        super().__init__(response=response)

                    @property
                    @api_response_property()
                    def result(self):
                        return Workflow.Result(self._from_json('Result'))

                return _Response(response=self._post(data=body))



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.application_base import ApplicationBaseAttributes


class CiscoCSSAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "CiscoCSS"
	certificate_file = Attribute('Certificate File')
	chain_cert = Attribute('Chain Cert')
	ftp_credential = Attribute('FTP Credential')
	ftp_host = Attribute('FTP Host')
	ftp_path = Attribute('FTP Path')
	ftp_port = Attribute('FTP Port')
	ftp_record = Attribute('FTP Record')
	ftp_type = Attribute('FTP Type')
	file_validation_disabled = Attribute('File Validation Disabled')
	network_validation_disabled = Attribute('Network Validation Disabled')
	private_key_file = Attribute('Private Key File')
	reuse_private_key = Attribute('Reuse Private Key')
	ssl_proxy_list = Attribute('SSL Proxy List')
	ssl_server = Attribute('SSL Server')
	server_type = Attribute('Server Type')
	temp_certificate_label = Attribute('Temp Certificate Label')



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.flow_action_approver_base import FlowActionApproverBaseAttributes


class FlowActionDirectApproversAttributes(FlowActionApproverBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Flow Action Direct Approvers"
	direct_approver = Attribute('Direct Approver', min_version='19.2')



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.top import TopAttributes


class CodeSigningApplicationAttributes(TopAttributes, metaclass=IterableMeta):
	__config_class__ = "Code Signing Application"
	application_hash = Attribute('Application Hash', min_version='19.2')
	application_location = Attribute('Application Location', min_version='19.2')
	application_signatory_issuer = Attribute('Application Signatory Issuer', min_version='19.2')
	application_signatory_subject = Attribute('Application Signatory Subject', min_version='19.2')
	application_size = Attribute('Application Size', min_version='19.2')
	application_version = Attribute('Application Version', min_version='19.2')
	permitted_argument_pattern = Attribute('Permitted Argument Pattern', min_version='19.2')
	regular_expression = Attribute('Regular Expression', min_version='21.1')
	signing_object_file_argument_pattern = Attribute('Signing Object File Argument Pattern', min_version='19.2')



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.code_signing_environment_base import CodeSigningEnvironmentBaseAttributes


class CodeSigningCertificateEnvironmentAttributes(CodeSigningEnvironmentBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Code Signing Certificate Environment"
	ca_specific_attributes = Attribute('CA Specific Attributes', min_version='21.3')
	certificate_authority = Attribute('Certificate Authority', min_version='20.3')
	certificate_dn = Attribute('Certificate DN', min_version='20.1')
	city = Attribute('City', min_version='20.3')
	country = Attribute('Country', min_version='20.3')
	elliptic_curve = Attribute('Elliptic Curve', min_version='20.3')
	key_algorithm = Attribute('Key Algorithm', min_version='20.3')
	key_bit_strength = Attribute('Key Bit Strength', min_version='20.3')
	key_storage_location = Attribute('Key Storage Location', min_version='20.3')
	organization = Attribute('Organization', min_version='20.3')
	organizational_unit = Attribute('Organizational Unit', min_version='20.3')
	provide_chain = Attribute('Provide Chain', min_version='20.2')
	state = Attribute('State', min_version='20.3')
	synchronize_chain = Attribute('Synchronize Chain')
	target_store = Attribute('Target Store', min_version='20.4')
	x509_subject = Attribute('X509 Subject', min_version='20.3')
	x509_subjectaltname_rfc822 = Attribute('X509 SubjectAltName RFC822', min_version='20.3')



from pytpp.properties.response_objects.dataclasses import permissions


class Permissions:
    @staticmethod
    def Permissions(response_object: dict):
        if not isinstance(response_object, dict):
            response_object = {}
        return permissions.Permissions(
            is_associate_allowed=response_object.get('IsAssociateAllowed'),
            is_create_allowed=response_object.get('IsCreateAllowed'),
            is_delete_allowed=response_object.get('IsDeleteAllowed'),
            is_manage_permissions_allowed=response_object.get('IsManagePermissionsAllowed'),
            is_policy_write_allowed=response_object.get('IsPolicyWriteAllowed'),
            is_private_key_read_allowed=response_object.get('IsPrivateKeyReadAllowed'),
            is_private_key_write_allowed=response_object.get('IsPrivateKeyWriteAllowed'),
            is_read_allowed=response_object.get('IsReadAllowed'),
            is_rename_allowed=response_object.get('IsRenameAllowed'),
            is_revoke_allowed=response_object.get('IsRevokeAllowed'),
            is_view_allowed=response_object.get('IsViewAllowed'),
            is_write_allowed=response_object.get('IsWriteAllowed'),
        )



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.application_base import ApplicationBaseAttributes
from pytpp.attributes.client_work_base import ClientWorkBaseAttributes
from pytpp.attributes.x509_certificate_base import X509CertificateBaseAttributes


class ServerCertificateWorkAttributes(ApplicationBaseAttributes, ClientWorkBaseAttributes, X509CertificateBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Server Certificate Work"
	application_attribute = Attribute('Application Attribute', min_version='18.3')
	application_type = Attribute('Application Type', min_version='18.3')
	certificate_container = Attribute('Certificate Container', min_version='18.3')
	friendly_name = Attribute('Friendly Name', min_version='18.3')
	interval = Attribute('Interval', min_version='18.3')
	log_threshold = Attribute('Log Threshold', min_version='18.3')
	naming_pattern = Attribute('Naming Pattern', min_version='18.3')
	nix_key_store = Attribute('Nix Key Store', min_version='18.3')
	nix_private_key = Attribute('Nix Private Key', min_version='18.3')
	path_type = Attribute('Path Type')
	private_key_trustee = Attribute('Private Key Trustee', min_version='20.2')
	win_key_store = Attribute('Win Key Store', min_version='18.3')
	win_private_key = Attribute('Win Private Key', min_version='18.3')



from pytpp.api.api_base import API, APIResponse, api_response_property
from pytpp.properties.response_objects.log import Log


class _Log(API):
    def __init__(self, api_obj):
        super().__init__(api_obj=api_obj, url='/Log')
        self.LogSchema = self._LogSchema(api_obj=api_obj)

    def get(self, component: str = None, from_time: str = None, grouping: int = None, id: int = None,
            limit: int = None, offset: int = None, order: str = None, severity: str = None, text1: str = None,
            text2: str = None, to_time: str = None, value1: str = None, value2: str = None):
        params = {
            'Component': component,
            'FromTime': from_time,
            'Grouping': grouping,
            'Id': id,
            'Limit': limit,
            'Offset': offset,
            'Order': order,
            'Severity': severity,
            'Text1': text1,
            'Text2': text2,
            'ToTime': to_time,
            'Value1': value1,
            'Value2': value2
        }
        
        class _Response(APIResponse):
            def __init__(self, response):
                super().__init__(response=response)

            @property
            @api_response_property()
            def log_events(self):
                return [Log.LogEvent(log) for log in self._from_json('LogEvents')]

        return _Response(response=self._get(params=params))

    def post(self, component: str, id: int, grouping: int = None, severity: int = None, source_ip: str = None,
             text1: str = None, text2: str = None, value1: str = None, value2: str = None):
        body = {
            'Component': component,
            'ID': id,
            'Grouping': grouping,
            'Severity': severity,
            'SourceIp': source_ip,
            'Text1': text1,
            'Text2': text2,
            'Value1': value1,
            'Value2': value2
        }

        class _Response(APIResponse):
            def __init__(self, response):
                super().__init__(response=response)

            @property
            @api_response_property()
            def log_result(self) -> int:
                return self._from_json('LogResult')

        return _Response(response=self._post(data=body))

    def Guid(self, guid: str):
        return self._Guid(guid=guid, api_obj=self._api_obj)

    class _Guid(API):
        def __init__(self, guid: str, api_obj):
            super().__init__(api_obj=api_obj, url=f'/Log/{guid}')

        def get(self):
            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def log_events(self):
                    return [Log.LogEvent(log) for log in self._from_json('LogEvents')]

            return _Response(response=self._get())

    class _LogSchema(API):
        def __init__(self, api_obj):
            super().__init__(api_obj=api_obj, url='Log/LogSchema')

        def get(self):
            class _Response(APIResponse):
                def __init__(self, response):
                    super().__init__(response=response)

                @property
                @api_response_property()
                def log_event_application_definitions(self):
                    return [Log.LogEventApplicationDefinition(lead) for lead in
                            self._from_json(key='LogEventApplicationDefinitions')]

                @property
                @api_response_property()
                def log_event_definitions(self):
                    return [Log.LogEventDefinition(led) for led in
                            self._from_json(key='LogEventDefinitions')]

                @property
                @api_response_property()
                def log_result(self) -> int:
                    return self._from_json(key='LogResult')

            return _Response(response=self._get())



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.application_base import ApplicationBaseAttributes


class CAPIAttributes(ApplicationBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "CAPI"
	binding_ip_address = Attribute('Binding IP Address')
	binding_port = Attribute('Binding Port')
	create_binding = Attribute('Create Binding')
	crypto_service_provider = Attribute('Crypto Service Provider', min_version='17.2')
	file_validation_disabled = Attribute('File Validation Disabled')
	friendly_name = Attribute('Friendly Name')
	hostname = Attribute('Hostname', min_version='15.4')
	network_validation_disabled = Attribute('Network Validation Disabled')
	non_exportable = Attribute('Non-Exportable')
	private_key_label = Attribute('Private Key Label', min_version='17.4')
	private_key_location = Attribute('Private Key Location', min_version='17.2')
	private_key_trustee = Attribute('Private Key Trustee')
	update_iis = Attribute('Update IIS')
	web_site_name = Attribute('Web Site Name')



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.connection_base import ConnectionBaseAttributes
from pytpp.attributes.driver_base import DriverBaseAttributes


class CertificateTrustStoreBaseAttributes(ConnectionBaseAttributes, DriverBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Certificate Trust Store Base"
	approver = Attribute('Approver')
	automatic_provisioning_disabled = Attribute('Automatic Provisioning Disabled')
	bundle = Attribute('Bundle')
	bundle_installed = Attribute('Bundle Installed')
	in_error = Attribute('In Error')
	interval = Attribute('Interval')
	stage = Attribute('Stage')
	status = Attribute('Status')



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes


class XolphinCAAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Xolphin CA"
	brand = Attribute('Brand')
	included_domains = Attribute('Included Domains')
	product = Attribute('Product')
	type = Attribute('Type')



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.certificate_authority_base import CertificateAuthorityBaseAttributes


class EntrustSecurityManagerCAAttributes(CertificateAuthorityBaseAttributes, metaclass=IterableMeta):
	__config_class__ = "Entrust Security Manager CA"
	certificate_type = Attribute('Certificate Type', min_version='15.3')
	create_entrust_user = Attribute('Create Entrust User')
	epf_credential = Attribute('EPF Credential')
	epf_credential_dn = Attribute('EPF Credential DN')
	enrollment_server_for_web_folder = Attribute('Enrollment Server for Web Folder')
	ini_file = Attribute('INI File')
	role = Attribute('Role', min_version='15.3')
	searchbase = Attribute('Searchbase')
	user_class_name = Attribute('User Class Name', min_version='15.3')



from dataclasses import dataclass
from datetime import datetime
from typing import List


@dataclass
class ResultCode:
    code: int
    codesign_result: str


@dataclass
class Items:
    items: List[str]


@dataclass
class InfoValue:
    info: int
    value: 'Items'


@dataclass
class CustomFieldAttributes:
    field_name: str
    values: List[str]


@dataclass
class EnvironmentTemplateDetails(InfoValue):
    template_values: 'InfoValue'


@dataclass
class RightsKeyValue:
    key: str
    value: int


@dataclass
class AppleTemplate:
    allow_user_key_import: bool
    cn_pattern: str
    dn: str
    guid: str
    id: int
    key_storage_location: 'InfoValue'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    read_only: bool
    target_policy_dn: str
    type: str
    visible_to: 'Items'


@dataclass
class AppleEnvironment:
    allow_user_key_import: bool
    apple_template: 'AppleTemplate'
    custom_field_attributes: 'List[CustomFieldAttributes]'
    dirty: bool
    dn: str
    guid: str
    id: int
    ip_address_restriction: 'Items'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    per_user: bool
    template_dn: str


@dataclass
class CertificateTemplate:
    allow_user_key_import: bool
    certificate_authority_dn: 'InfoValue'
    certificate_subject: str
    city: 'InfoValue'
    country: 'InfoValue'
    dn: str
    guid: str
    id: int
    key_algorithm: 'InfoValue'
    key_storage_location: 'InfoValue'
    key_use_flow_dn: str
    object_naming_pattern: str
    organization: 'InfoValue'
    organization_unit: 'InfoValue'
    per_user: bool
    read_only: bool
    san_email: 'InfoValue'
    state: 'InfoValue'
    target_policy_dn: str
    type: str
    visible_to: 'Items'


@dataclass
class CertificateEnvironment:
    allow_user_key_import: bool
    ca_specific_attributes: 'Items'
    certificate_authority_dn: 'InfoValue'
    certificate_subject: str
    certificate_template: 'CertificateTemplate'
    city: 'InfoValue'
    country: 'InfoValue'
    custom_field_attributes: 'List[CustomFieldAttributes]'
    dn: str
    guid: str
    id: int
    ip_address_restriction: 'Items'
    key_algorithm: 'InfoValue'
    key_storage_location: 'InfoValue'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    organization: 'InfoValue'
    organization_unit: 'InfoValue'
    per_user: bool
    san_email: 'InfoValue'
    state: 'InfoValue'
    status: int
    target_store: str
    template_dn: str
    type: str


@dataclass
class CSPTemplate:
    allow_user_key_import: bool
    dirty: bool
    dn: str
    encryption_key_algorithm: 'InfoValue'
    expiration: 'InfoValue'
    guid: str
    id: int
    key_container_dn: str
    key_storage_location: 'InfoValue'
    max_uses: 'InfoValue'
    object_naming_pattern: str
    per_user: bool
    signing_key_algorithm: 'InfoValue'
    type: str
    visible_to: 'Items'


@dataclass
class CSPEnvironment:
    allow_user_key_import: bool
    csp_template: 'CSPTemplate'
    disabled: bool
    dn: str
    encryption_key_algorithm:'EnvironmentTemplateDetails'
    encryption_key_dn: str
    expiration: int
    guid: str
    id: int
    ip_address_restriction: 'Items'
    key_storage_location: 'EnvironmentTemplateDetails'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    max_uses: int
    per_user: bool
    signing_key_algorithm: 'EnvironmentTemplateDetails'
    signing_key_dn: str
    template_dn: str


@dataclass
class DotNetTemplate:
    allow_user_key_import: bool
    disabled: bool
    dn: str
    expiration: 'InfoValue'
    guid: str
    id: int
    key_algorithm: 'InfoValue'
    key_container_dn: 'InfoValue'
    key_storage_location: 'InfoValue'
    key_use_flow_dn: str
    max_uses: 'InfoValue'
    object_naming_pattern: str
    per_user: bool
    type: str
    visible_to: 'Items'


@dataclass
class DotNetEnvironment:
    allow_user_key_import: bool
    dot_net_template: 'DotNetTemplate'
    disabled: bool
    dn: str
    expiration: int
    guid: str
    id: int
    ip_address_restriction: 'Items'
    key_algorithm: 'EnvironmentTemplateDetails'
    key_dn: str
    key_storage_location: 'EnvironmentTemplateDetails'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    max_uses: int
    per_user: bool
    template_dn: str


@dataclass
class GPGTemplate:
    allow_user_key_import: bool
    authentication_key_algorithm: 'InfoValue'
    dn: str
    email: 'InfoValue'
    encryption_key_algorithm: 'InfoValue'
    expiration: 'InfoValue'
    guid: str
    id: int
    key_storage_location: 'InfoValue'
    key_use_flow_dn: str
    max_uses: 'InfoValue'
    object_naming_pattern: str
    per_user: bool
    read_only: bool
    real_name: 'InfoValue'
    signing_key_algorithm: 'InfoValue'
    type: str
    visible_to: 'Items'


@dataclass
class GPGEnvironment:
    allow_user_key_import: bool
    authentication_key_algorithm: 'EnvironmentTemplateDetails'
    custom_fields_attributes: 'List[CustomFieldAttributes]'
    dirty: bool
    dn: str
    email: List[str]
    encryption_key_algorithm: 'EnvironmentTemplateDetails'
    expiration: int
    guid: str
    gpg_template: 'GPGTemplate'
    id: int
    ip_address_restriction: 'Items'
    key_storage_location: 'EnvironmentTemplateDetails'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    max_uses: int
    per_user: bool
    real_name: 'EnvironmentTemplateDetails'
    read_only: bool
    signing_key_algorithm: 'EnvironmentTemplateDetails'
    status: int
    template_dn: str
    type: str


@dataclass
class KeyPairTemplate:
    allow_user_key_import: bool
    description: str
    dirty: bool
    dn: str
    expiration: 'InfoValue'
    guid: str
    id: int
    key_algorithm:'InfoValue'
    key_container_dn: str
    key_storage_location: 'InfoValue'
    max_uses: int
    type: str
    validity_period: int
    visible_to: 'Items'


@dataclass
class KeyPairEnvironment:
    allow_user_key_import: bool
    custom_fields_attributes: 'List[CustomFieldAttributes]'
    dn: str
    expiration: int
    guid: str
    id: int
    ip_address_restriction: 'Items'
    key_algorithm: 'InfoValue'
    key_dn: str
    key_pair_template: 'KeyPairTemplate'
    key_storage_location: 'EnvironmentTemplateDetails'
    key_time_constraints: 'Items'
    key_use_flow_dn: str
    status: int
    template_dn: str
    type: str


@dataclass
class KeyStorageLocations:
    items: List[str]


@dataclass
class Application:
    dn: str
    guid: str
    id: int


@dataclass
class ApplicationCollection:
    application_dns: 'Items'
    dn: str
    guid: str
    id: int


@dataclass
class GlobalConfiguration:
    approved_key_storage_locations: 'KeyStorageLocations'
    available_key_storage_locations: 'KeyStorageLocations'
    default_ca_container: str
    default_certificate_container: str
    default_credential_container: str
    key_use_timeout: int
    project_description_tooltip: str
    request_in_progress_message: str


@dataclass
class Project:
    application_dns: 'Items'
    applications: 'List[Application]'
    auditors: 'Items'
    certificate_environments: 'List[CertificateEnvironment]'
    collections: 'List[SignApplicationCollection]'
    created_on: datetime
    description: str
    dn: str
    guid: str
    id: int
    key_use_approvers: 'Items'
    key_users: 'Items'
    owners: 'Items'
    status: int


@dataclass
class Rights:
    none: int
    admin: int
    use: int
    audit: int
    owner: int
    project_approval: int
    application_admin: int
    approve_use: int


@dataclass
class SignApplicationCollection:
    description: str
    dn: str
    guid: str
    hash: str
    id: int
    location: str
    permitted_argument_pattern: str
    signatory_issuer: str
    signatory_subject: str
    size: int
    version: str



from pytpp.attributes._helper import IterableMeta, Attribute
from pytpp.attributes.top import TopAttributes


class LogApplicationAttributes(TopAttributes, metaclass=IterableMeta):
	__config_class__ = "Log Application"
	configuration = Attribute('Configuration')
	log_application_id = Attribute('Log Application ID')
	log_application_name = Attribute('Log Application Name')
	log_application_schema_en = Attribute('Log Application Schema EN')
